from contextlib import asynccontextmanager
from datetime import datetime
from pathlib import Path

from fastapi import FastAPI
from py_tools.connections.http import AsyncHttpClient
from py_tools.logging import logger

from starlette.staticfiles import StaticFiles

from .config.site import SiteConfig
from .middlewares.base import register_middlewares

from .router.graph_api import graph_router
from .router.knowledge_nodes_api import knowledge_nodes_api_router
from .router.knowledge_saas import saas_kb_router
from .router.text_search import text_search_router

from .utils import log_util



@asynccontextmanager
async def lifespan(app: FastAPI):
    await startup()
    yield
    await shutdown()

config = SiteConfig()
app = FastAPI(
    description="知识图谱开放平台",
    lifespan=lifespan,
    middleware=register_middlewares(),  # 注册web中间件
)
@app.get("/health")
async def health_check():
    """健康检查接口"""
    return {
        "status": "ok",
        "timestamp": datetime.utcnow().isoformat(),
        "service": "knowledge-graph"
    }



async def init_setup():
    """初始化项目配置"""

    log_util.setup_logger()
    
async def startup():
    """项目启动时准备环境"""

    await init_setup()

    # 加载路由
    app.include_router(knowledge_nodes_api_router)
    app.include_router(text_search_router)
    app.include_router(graph_router)
    app.include_router(saas_kb_router)

    # 挂载静态文件目录，将/books路径映射到本地books文件夹

    books_path = Path(config.get_config("BOOKS"))
    #books_path = Path("E:\\project\\knowledge\\books")

    app.mount("/books", StaticFiles(directory=books_path), name="books")

    # 需要拦截的 URL 列表（支持通配符）
    INTERCEPT_URLS = {
        "/v1/knowledge/*"
    }

    # 白名单 URL（不需要拦截的路径）
    WHITE_LIST = {
        "/books/*",
        "/knowledge/*"
    }
    logger.info("fastapi startup success")


async def shutdown():
    await AsyncHttpClient.close()
    logger.error("app shutdown")
