# 导入FastAPI及相关模块
import json

import uvicorn
from fastapi import FastAPI,Query
from fastapi_mcp import FastApiMCP
from py_tools.logging import logger

from src.knowledge.middlewares.base import register_middlewares
from .settings import base_setting

from contextlib import asynccontextmanager
from py_tools.logging import logger
from datetime import datetime

@asynccontextmanager
async def lifespan(app: FastAPI):
    await startup(app)
    yield
    await shutdown()
app = FastAPI(
    description="知识图谱开放平台",
    lifespan=lifespan,
    middleware=register_middlewares(),  # 注册web中间件
)


async def startup(app: FastAPI):
    """项目启动时准备环境"""
    from py_tools.logging import logger
    from pathlib import Path
    from starlette.staticfiles import StaticFiles
    from .config.site import SiteConfig
    from .router.graph_api import graph_router
    from .router.knowledge_nodes_api import knowledge_nodes_api_router
    from .router.knowledge_saas import saas_kb_router
    from .router.text_search import text_search_router
    from .utils import log_util
    
    log_util.setup_logger()
    
    # 加载路由
    app.include_router(knowledge_nodes_api_router)
    app.include_router(text_search_router)
    app.include_router(graph_router)
    app.include_router(saas_kb_router)
    #app.include_router(medical_knowledge_router)
    # 挂载静态文件目录
    config = SiteConfig()
    books_path = Path(config.get_config("BOOKS"))
    app.mount("/books", StaticFiles(directory=books_path), name="books")
    static_dir = Path(__file__).parent / "static"
    app.mount("/static", StaticFiles(directory=str(static_dir)), name="static")
    
    logger.info("fastapi startup success")

async def shutdown():
    from py_tools.connections.http import AsyncHttpClient
    from py_tools.logging import logger
    await AsyncHttpClient.close()
    logger.error("app shutdown")


# @app.get("/health")
# async def health_check():
#     """健康检查接口"""
#     return {
#         "status": "ok",
#         "timestamp": datetime.utcnow().isoformat(),
#         "service": "knowledge-graph"
#     }

from fastapi import APIRouter, Depends, HTTPException
from pydantic import BaseModel
from typing import List, Optional, Dict, Any


from .model.response import StandardResponse
from .db.session import get_db
from sqlalchemy.orm import Session
import logging


from .service.search_service import SearchBusiness
from .utils.MedicalRecordParser import MedicalRecordParser
from pydantic import BaseModel
from typing import Optional

logger = logging.getLogger(__name__)

# 1. 疾病与症状相关接口
class SymptomDiseasesRequest(BaseModel):
    symptoms: List[str]


class DiseaseSymptomsRequest(BaseModel):
    disease_id: str


class DiseaseInfoRequest(BaseModel):
    query: str
    type: Optional[str] = None


@app.post("/symptom_diseases", response_model=StandardResponse, operation_id="getSymptomDiseases",
          summary="根据症状获取相关疾病列表",
          description="""根据输入的症状列表，查询可能相关的疾病列表。

该接口主要用于医疗知识查询场景，例如：
- 通过输入症状名称查询相关疾病
- 了解症状可能对应的疾病范围
- 辅助临床诊断决策

典型应用场景：
1. 初步诊断：输入患者症状获取可能疾病
2. 鉴别诊断：通过症状缩小疾病范围
3. 健康教育：了解症状可能对应的疾病

输入要求：
- 症状名称应为标准医学术语

输出格式：
- 返回标准JSON响应
- 包含疾病列表""",
          response_description="""返回标准响应格式，包含疾病名称列表。

成功响应示例：
{
    "success": true,
    "data": ["流感", "肺炎"]
}

错误响应示例：
{
    "success": false,
    "error": "Invalid symptom name"
}""")
async def get_symptom_diseases(
        symptoms: List[str] = Query(...,
                  alias="symptomNames",
                  description="""症状名称列表。

该参数用于查询与输入症状相关的疾病列表。
建议使用标准医学术语。
示例值：["头痛", "发热"]""",
                  examples=[["头痛", "发热"]],
                  min_length=1,
                  max_length=10)
):

    try:
        #打印request
        logger.info(f"symptoms: {symptoms}")
        # 实现获取症状相关疾病的逻辑
        search = SearchBusiness()
        results = search.get_symptom_diseases(symptoms)
        # 直接返回疾病名称列表
        disease_names = [disease["disease_name"] for disease in results] if results else []
        return StandardResponse(success=True, data=disease_names)
    except Exception as e:
        logger.exception(f"获取症状相关疾病失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))


@app.post("/get_lab_examinations", 
          response_model=StandardResponse, 
          operation_id="getLabExaminations",
          summary="根据疾病名称获取相关实验室检查",
          description="""根据疾病名称获取该疾病的相关实验室检查项目。

该接口主要用于医疗知识查询场景，例如：
- 通过输入疾病名称查询其相关实验室检查
- 了解疾病的常规检验项目
- 辅助临床检验决策

典型应用场景：
1. 临床检验辅助：通过疾病名称查询其相关实验室检查
2. 检验科参考：了解疾病应该做哪些实验室检查
3. 医学知识问答系统：为AI提供结构化实验室检查数据

输入要求：
- 仅支持标准疾病名称
- 不支持症状描述或编码

输出格式：
- 返回标准JSON响应
- 包含实验室检查项目列表""",
          response_description="""返回标准响应格式，包含实验室检查项目列表。

成功响应示例：
{
    "success": true,
    "data": {
        "disease": "糖尿病",
        "lab_examinations": [
            "空腹血糖",
            "糖化血红蛋白"
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid disease name"
}"""
          )
async def get_lab_examinations(
        disease_name: str = Query(..., 
                  alias="diseaseName", 
                  description="""疾病名称（仅支持标准疾病名称）。

该参数用于查询该疾病的相关实验室检查项目。
必须使用标准疾病名称。
示例值："糖尿病""",
                  examples=["糖尿病"],
                  min_length=1,
                  max_length=50)
):
    try:
        search = SearchBusiness()
        results = search.get_lab_examinations(disease_name=disease_name)
        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"实验室检查查询失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))


@app.post("/get_auxiliary_examinations", 
          response_model=StandardResponse, 
          operation_id="getAuxiliaryExaminations",
          summary="根据疾病名称获取相关辅助检查",
          description="""根据疾病名称获取该疾病的相关辅助检查项目。

该接口主要用于医疗知识查询场景，例如：
- 通过输入疾病名称查询其相关辅助检查
- 了解疾病的常规检查项目
- 辅助临床检查决策

典型应用场景：
1. 临床检查辅助：通过疾病名称查询其相关辅助检查
2. 影像科参考：了解疾病应该做哪些辅助检查
3. 医学知识问答系统：为AI提供结构化辅助检查数据

输入要求：
- 仅支持标准疾病名称
- 不支持症状描述或编码

输出格式：
- 返回标准JSON响应
- 包含辅助检查项目列表""",
          response_description="""返回标准响应格式，包含辅助检查项目列表。

成功响应示例：
{
    "success": true,
    "data": {
        "disease": "冠心病",
        "auxiliary_examinations": [
            "心电图",
            "冠状动脉造影"
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid disease name"
}"""
          )
async def get_auxiliary_examinations(
        disease_name: str = Query(..., 
                  alias="diseaseName", 
                  description="""疾病名称（仅支持标准疾病名称）。

该参数用于查询该疾病的相关辅助检查项目。
必须使用标准疾病名称。
示例值："冠心病""",
                  examples=["冠心病"],
                  min_length=1,
                  max_length=50)
):
    try:
        search = SearchBusiness()
        results = search.get_auxiliary_examinations(disease_name=disease_name)
        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"辅助检查查询失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))

@app.post("/getDiseaseSymptoms",
          response_model=StandardResponse,
          operation_id="getDiseaseSymptoms",
          summary="根据疾病名称获取相关症状",
          description="""根据疾病名称获取该疾病的所有症状。

该接口主要用于医疗知识查询场景，例如：
- 通过输入疾病名称查询相关症状
- 了解疾病的主要临床表现
- 辅助临床诊断决策

典型应用场景：
1. 临床诊断辅助：通过疾病名称获取典型症状
2. 患者教育：了解疾病可能出现的症状
3. 医学知识问答系统：为AI提供结构化症状数据

输入要求：
- 仅支持标准医学术语名称
- 不支持任何形式的编码（如ICD-10等）

输出格式：
- 返回标准JSON响应
- 包含症状列表""",
          response_description="""返回标准响应格式，包含症状列表。

成功响应示例：
{
    "success": true,
    "data": {
        "disease": "胃癌",
        "symptoms": [
            "胃丢失蛋白",
            "胃肠粘膜的通透性增高",
            "手掌呈黑色",
            "上腹痛",
            "吞咽困难",
            "疲劳",
            "进食后饱胀",
            "贫血",
            "黑便",
            "恶心"
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid disease name"
}"""
          )
async def getDiseaseSymptoms(
        disease_name: str = Query(...,
                  alias="diseaseName",
                  description="""疾病名称（仅支持标准医学术语名称，不支持编码）。

该参数用于查询该疾病的所有症状及其严重程度权重。
必须使用标准医学术语名称。
示例值："胃癌""",
                  examples=["胃癌"],
                  min_length=1,
                  max_length=50)
):

    try:
        # 验证疾病名称有效性
        if not disease_name or len(disease_name.strip()) == 0:
            raise ValueError("疾病名称不能为空")
            
        # 调用业务逻辑获取症状
        search = SearchBusiness()
        results = search.get_disease_symptoms(disease_name)
        return StandardResponse(success=True, data=results)
    except ValueError as e:
        logger.warning(f"无效的疾病名称: {disease_name}")
        raise HTTPException(400, detail=StandardResponse.error(str(e)))
    except Exception as e:
        logger.error(f"获取疾病症状失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))


# @app.post("/disease_info", response_model=StandardResponse)
# async def get_disease_info(
#         request: DiseaseInfoRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现搜索医学概念的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"搜索医学概念失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))
#
#
# 2. 药物相关接口

@app.post("/drug_indications", 
          response_model=StandardResponse, 
          operation_id="getDrugIndications",
          summary="根据药物名称获取适应症",
          description="""根据药物名称获取该药物的所有适应症。

该接口主要用于医疗知识查询场景，例如：
- 通过输入药物名称查询其适应症
- 了解药物的主要治疗范围
- 辅助临床用药决策

典型应用场景：
1. 临床用药辅助：通过药物名称查询其适应症
2. 患者教育：了解药物的治疗范围
3. 医学知识问答系统：为AI提供结构化药物适应症数据

输入要求：
- 仅支持标准药物名称
- 不支持商品名或编码

输出格式：
- 返回标准JSON响应
- 包含适应症列表""",
          response_description="""返回标准响应格式，包含适应症列表。

成功响应示例：
{
    "success": true,
    "data": {
        "drug": "阿司匹林",
        "indications": [
            "镇痛",
            "解热",
            "抗炎",
            "抗风湿",
            "预防心血管事件"
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid drug name"
}"""
          )
async def get_drug_indications(
        drug_name: str = Query(..., 
                  alias="drugName", 
                  description="""药物名称（仅支持标准药物名称，不支持商品名）。

该参数用于查询该药物的所有适应症。
必须使用标准药物名称。
示例值："阿司匹林""",
                  examples=["阿司匹林"],
                  min_length=1,
                  max_length=50)
):
    try:
        # 实现获取药物适应症的逻辑
        search = SearchBusiness()
        results = search.get_drug_indications(drug_name)
        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"获取药物适应症失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))


# @app.post("/drug_contraindications", response_model=StandardResponse)
# async def get_drug_contraindications(
#         request: DrugRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现获取药物禁忌症的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"获取药物禁忌症失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))


# @app.post("/drug_interactions", response_model=StandardResponse)
# async def get_drug_interactions(
#         request: DrugRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现获取药物相互作用的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"获取药物相互作用失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))


# 3. 关系与概念处理接口
@app.post("/search_concept",
          response_model=StandardResponse, 
          operation_id="searchConcept",
          summary="根据名称和类型搜索医学概念",
          description="""根据概念名称和类型搜索医学概念节点。

该接口主要用于医疗知识图谱中的概念搜索场景，例如：
- 通过输入疾病名称和类型搜索相关疾病概念
- 通过输入症状名称和类型搜索相关症状概念
- 通过输入药物名称和类型搜索相关药物概念

典型应用场景：
1. 疾病概念搜索：输入疾病名称获取相关概念
2. 症状概念搜索：输入症状名称获取相关概念
3. 药物概念搜索：输入药物名称获取相关概念

输入要求：
- 概念名称应为标准医学术语
- 类型参数可选，用于限定搜索范围

输出格式：
- 返回标准JSON响应
- 包含匹配的概念列表""",
          response_description="""返回标准响应格式，包含匹配的概念列表。

成功响应示例：
{
    "success": true,
    "data": {
        "concepts": [
            {"name": "糖尿病", "type": "疾病"},
            {"name": "高血压", "type": "疾病"}
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid concept name"
}"""
          )
async def search_concept(
        concept_name: str = Query(...,
                  description="概念名称，应为标准医学术语。该参数用于搜索匹配的概念节点。",
                  examples=["糖尿病"],
                  min_length=1),
        concept_type: str = Query(None,
                  description="概念类型，可选参数。用于限定搜索的概念类型。",
                  examples=["疾病", "症状", "药物"])
):
    try:
        # 实现搜索医学概念的逻辑
        search = SearchBusiness()
        results = search.search_concept(name=concept_name, type=concept_type)
        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"搜索医学概念失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))


@app.post("/get_relations", 
          response_model=StandardResponse,
          operation_id="getConceptRelations",
          summary="根据概念名称查询相关关系",
          description="""根据概念名称查询该概念的相关关系列表。

该接口主要用于医疗知识图谱中的关系查询场景，例如：
- 通过输入疾病概念名称查询相关症状
- 通过输入症状概念名称查询相关疾病
- 通过输入药物概念名称查询适应症

典型应用场景：
1. 疾病症状关系查询：输入疾病名称获取相关症状
2. 症状疾病关系查询：输入症状名称获取相关疾病
3. 药物适应症查询：输入药物名称获取适应症

输入要求：
- 概念名称应为标准医学术语

输出格式：
- 返回标准JSON响应
- 包含相关关系列表""",
          response_description="""返回标准响应格式，包含相关关系列表。

成功响应示例：
{
    "success": true,
    "data": {
        "relations": [
            {"relation_type": "has_symptom", "target_id": "S001", "target_name": "发热"},
            {"relation_type": "has_symptom", "target_id": "S002", "target_name": "咳嗽"}
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid concept ID"
}"""
          )
async def get_relations(
        concept_name: str = Query(...,
                  description="概念ID，应为标准医学术语ID。该参数用于查询该概念的相关关系。",
                  examples=["糖尿病"],
                  min_length=1),
        concept_type: str = Query(None,
                  description="概念类型，可选参数。用于限定搜索的概念类型。",
                  examples=["疾病", "症状", "药物"])          
):
    try:
        # 实现获取概念关系的逻辑
        search = SearchBusiness()
        results = search.get_relations(name=concept_name, type=concept_type)
        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"获取概念关系失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))

# @app.post("/parse_medical_record",
#     response_model=StandardResponse,
#     operation_id="parseMedicalRecord",
#     summary="病历文本结构化",
#     description="""将病历文本解析为结构化数据。
#
# 该接口主要用于医疗病历处理场景，例如：
# - 将非结构化的病历文本转换为结构化JSON数据
# - 提取病历中的关键医疗信息
# - 辅助临床决策支持系统
#
# 典型应用场景：
# 1. 电子病历系统：将医生书写的病历文本结构化存储
# 2. 临床研究：从病历中提取标准化数据用于分析
# 3. 医疗AI：为自然语言处理模型提供结构化输入
#
# 输入要求：
# - 病历文本应为中文医疗记录
# - 包含完整的病历信息（主诉、现病史等）
#
# 输出格式：
# - 返回标准JSON响应
# - 包含结构化后的病历数据""",
#     response_description="""返回标准响应格式，包含结构化后的病历数据。
#
# 成功响应示例：
# {
#     "success": true,
#     "data": {
#         "病历基本信息": {
#             "就诊日期": "2023-05-10",
#             "就诊科室": "心血管内科",
#             "门诊类型": "初诊"
#         },
#         "患者信息": {
#             "性别": "男",
#             "年龄": "45岁"
#         }
#     }
# }
#
# 错误响应示例：
# {
#     "success": false,
#     "error": "Invalid medical record text"
# }"""
# )
# async def parse_medical_record(
#     record_text: str = Query(...,
#                   description="""病历文本内容。
#
# 该参数应为完整的病历文本，包含主诉、现病史等关键部分。
# 示例值："患者男性，45岁，主诉胸闷3天...""",
#                   examples=["患者男性，45岁，主诉胸闷3天，加重1天..."]
# )):
#     try:
#         parser = MedicalRecordParser()
#         result = parser.parse_medical_record(record_text)
#         return StandardResponse(success=True, data=result)
#     except Exception as e:
#         logger.error(f"病历解析失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))


@app.post("/get_similar_concepts", 
          response_model=StandardResponse,
          operation_id="getSimilarConcepts",
          summary="根据概念名称查询相似概念",
          description="""根据概念名称查询该概念的相似概念列表。

该接口主要用于医疗知识图谱中的相似概念查询场景，例如：
- 通过输入疾病名称查询相似疾病概念
- 通过输入症状名称查询相似症状概念
- 通过输入药物名称查询相似药物概念

典型应用场景：
1. 疾病概念扩展：输入疾病名称获取相关相似疾病
2. 症状概念扩展：输入症状名称获取相关相似症状
3. 药物概念扩展：输入药物名称获取相关相似药物

输入要求：
- 概念名称应为标准医学术语
- 可指定返回相似概念的数量限制

输出格式：
- 返回标准JSON响应
- 包含相似概念列表""",
          response_description="""返回标准响应格式，包含相似概念列表。

成功响应示例：
{
    "success": true,
    "data": {
        "similar_concepts": [
            "艾滋病",
            "成人艾滋病",
            "儿童艾滋病"
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid concept name"
}"""
          )
async def get_similar_concepts(
        concept_name: str = Query(...,
                  description="概念名称，应为标准医学术语。该参数用于查询该概念的相似概念。",
                  examples=["艾滋病"],
                  min_length=1),
        top_k: int = Query(5,
                  description="返回相似概念的数量限制，默认为5。",
                  ge=1, le=20)
):
    try:
        # 1. 先根据ID获取节点名称
        search = SearchBusiness()
        results = search.get_similar_concepts(name=concept_name, top_k=top_k)

        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"获取相似概念失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))


# 4. 临床辅助决策接口
class DiagnosisCheckRequest(BaseModel):
    diagnosis: str
    symptoms: List[str]
    lab_results: Optional[Dict[str, Any]] = None


class NextQuestionsRequest(BaseModel):
    current_input: Dict[str, Any]


class TriageRequest(BaseModel):
    symptoms: List[str]
    vital_signs: Dict[str, Any]


class CodingRequest(BaseModel):
    clinical_term: str


class DepartmentRequest(BaseModel):
    diagnosis: str
    symptoms: Optional[List[str]] = None


# @app.post("/medical/check_diagnosis", response_model=StandardResponse)
# async def check_diagnosis_consistency(
#         request: DiagnosisCheckRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现诊断一致性检查的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"诊断一致性检查失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))
#
#
# @app.post("/medical/suggest_questions", response_model=StandardResponse)
# async def suggest_next_questions(
#         request: NextQuestionsRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现建议下一步问题的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"建议下一步问题失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))
#
#
# @app.post("/medical/triage_guidance", response_model=StandardResponse)
# async def get_triage_guidance(
#         request: TriageRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现分诊建议的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"获取分诊建议失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))
#
#
# @app.post("/medical/coding_recommendation", response_model=StandardResponse)
# async def get_coding_recommendation(
#         request: CodingRequest,
#         db: Session = Depends(get_db)
# ):
#     try:
#         # 实现编码推荐的逻辑
#         return StandardResponse(success=True, data=[])
#     except Exception as e:
#         logger.error(f"获取编码推荐失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))
#
#
@app.post("/suggest_appropriate_department", 
          response_model=StandardResponse, 
          operation_id="suggestAppropriateDepartment",
          summary="根据疾病名称获取推荐科室",
          description="""根据疾病名称获取该疾病的推荐科室。

该接口主要用于医疗知识查询场景，例如：
- 通过输入疾病名称查询其推荐就诊科室
- 了解疾病的专科归属
- 辅助临床分诊决策

典型应用场景：
1. 临床分诊辅助：通过疾病名称查询其推荐科室
2. 患者就医指导：了解疾病应该挂哪个科室
3. 医学知识问答系统：为AI提供结构化疾病科室推荐数据

输入要求：
- 仅支持标准疾病名称
- 不支持症状描述或编码

输出格式：
- 返回标准JSON响应
- 包含推荐科室列表""",
          response_description="""返回标准响应格式，包含推荐科室列表。

成功响应示例：
{
    "success": true,
    "data": {
        "disease": "艾滋病",
        "departments": [
            "感染科",
            "皮肤性病科"
        ]
    }
}

错误响应示例：
{
    "success": false,
    "error": "Invalid disease name"
}"""
          )
async def suggest_appropriate_department(
        disease_name: str = Query(..., 
                  alias="diseaseName", 
                  description="""疾病名称（仅支持标准疾病名称）。

该参数用于查询该疾病的推荐科室。
必须使用标准疾病名称。
示例值："艾滋病""",
                  examples=["艾滋病"],
                  min_length=1,
                  max_length=50)
):
    try:
        # 实现科室推荐的逻辑
        search = SearchBusiness()
        results = search.suggest_appropriate_department(disease_name=disease_name)
        return StandardResponse(success=True, data=results)
    except Exception as e:
        logger.error(f"科室推荐失败: {str(e)}")
        raise HTTPException(500, detail=StandardResponse.error(str(e)))
#
#
# # 5. 病历质控接口
# class MedicalRecordRequest(BaseModel):
#     medical_record: Dict[str, Any]
#     validate_type: str
#
#


# @app.post("/medical/validate_record", response_model=StandardResponse, operation_id="validateMedicalRecord",
#           summary="验证病历内容的逻辑一致性",
#           description="""根据输入的病历内容，验证其逻辑一致性并返回验证结果。
#
# 该接口主要用于医疗病历质控场景，例如：
# - 验证诊断与症状的相关性
# - 检查病历中的逻辑矛盾
# - 确保病历符合医疗规范
#
# 典型应用场景：
# 1. 病历自动质控：输入病历内容获取验证结果
# 2. 临床决策支持：验证诊断与症状的匹配度
# 3. 病历完整性检查：确保病历包含必要信息
#
# 输入要求：
# - 病历内容应为标准医疗文本
#
# 输出格式：
# - 返回标准JSON响应
# - 包含验证结果和错误列表""",
#           response_description="""返回标准响应格式，包含验证结果。
#
# 成功响应示例：
# {
#     "success": true,
#     "data": {
#         "errors": [
#             "初步诊断'胆总管结石伴胆管炎'与主诉症状不相关",
#             "初步诊断'胆囊结石并胆囊炎'与主诉症状不相关"
#         ]
#     }
# }
#
# 错误响应示例：
# {
#     "success": false,
#     "error": "Invalid medical content"
# }""")
# async def validate_medical_record(
#         medical_content: str = Query(...,
#                                     description="""病历内容文本，应为标准医疗术语。
#
# 该参数用于验证病历的逻辑一致性。
# 必须使用标准医学术语。
# 示例值："患者主诉头痛3天，初步诊断：高血压病""",
#                                     min_length=10)
# ):
#     try:
#         # 调用search_service中的验证方法
#         search = SearchBusiness()
#         results = search.validate_medical_record(medical_content)
#         #只返回errors字段
#         results = results["errors"]
#         return StandardResponse(success=True, data=results)
#     except ValueError as e:
#         logger.warning(f"无效的病历内容: {medical_content}")
#         raise HTTPException(400, detail=StandardResponse.error(str(e)))
#     except Exception as e:
#         logger.error(f"病历验证失败: {str(e)}")
#         raise HTTPException(500, detail=StandardResponse.error(str(e)))


mcp = FastApiMCP(app)
mcp.mount()
mcp.setup_server()

def main():
    logger.info(f"project run {base_setting.server_host}:{base_setting.server_port}")
    uvicorn.run(
        app=app, host=base_setting.server_host, port=base_setting.server_port, log_level=base_setting.server_log_level,
        access_log=False
    )

if __name__ == "__main__":
    main()

