import json
from typing import Dict, Any
from .DeepseekUtil import chat

class MedicalRecordParser:
    """
    病历信息结构化工具类
    使用Deepseek大模型进行病历文本解析和结构化
    """

    @staticmethod
    def parse_medical_record(record_text: str) -> Dict[str, Any]:
        """
        解析病历文本并返回结构化数据
        
        Args:
            record_text: 原始病历文本
            
        Returns:
            结构化后的病历数据字典
        """
        # 构建提示词
        prompt = """你是一位专业的医疗信息处理助手，请将以下门诊病历文本严格按照JSON格式提取并结构化，要求如下：

1. 逐项提取对应字段，确保无遗漏
2. 空白字段保留空字符串或空数组
3. 时间格式统一为"YYYY-MM-DD"
4. 药品用法用量需完整提取规格信息
5. 诊断分类需明确区分初步/确诊/鉴别诊断
6. 专科检查需注明具体检查部位

输出格式要求：
```json
{
  "病历基本信息": {
    "就诊日期": "识别文本中的日期信息",
    "就诊科室": "",
    "门诊类型": "根据'初诊''复诊'关键词判断",
    "医生信息": {
      "姓名": "",
      "职称": "主任医师/副主任医师等",
      "科室": ""
    }
  },
  "患者信息": {
    "性别": "男/女/其他",
    "出生日期": "",
    "年龄": "自动计算或提取",
    "婚姻状况": "未婚/已婚/离异等",
    "职业": ""
  },
  "主诉": {
    "症状描述": "提取患者自述症状",
    "持续时间": "如'3天''2周'等",
    "主要症状": ["医生记录的核心症状"],
    "伴随症状": [],
    "加重因素": ""
  },
  "现病史": {
    "起病情况": "急性/慢性/隐匿等",
    "主要症状": ["医生记录的核心症状"],
    "伴随症状": [],
    "病情演变": "加重/缓解/反复等",
    "诊疗经过": "既往治疗史",
    "一般情况": "饮食/睡眠/二便等"
  },
  "体格检查": {
    "生命体征": {
      "体温": "带单位如36.5℃",
      "脉搏": "次/分",
      "呼吸": "次/分",
      "血压": "mmHg格式"
    },
    "专科检查": {
      "检查部位": "如'心肺''腹部'等",
      "检查结果": "描述性内容"
    }
  },
"辅助检查": {
    "实验室检查": [
      {
        "检查项目": "如'血常规''尿常规'等",
        "检查时间": "",
        "检查结果": {
          "数值结果": "",
          "文本结果": "",
          "异常标志": "正常/异常"
        },
        "参考范围": ""
      }
    ],
    "影像学检查": [
      {
        "检查类型": "如'胸部X线''腹部CT'等",
        "检查时间": "",
        "检查结果": "",
        "影像编号": ""
      }
    ],
    "病理检查": [
      {
        "检查类型": "",
        "取材部位": "",
        "报告时间": "",
        "病理诊断": ""
      }
    ],
    "其他检查": [
      {
        "检查名称": "如'心电图''肺功能'等",
        "检查结果": ""
      }
    ],
    "检查结论": "对异常结果的总结性描述"
  },
  "诊断信息": {
    "初步诊断": ["按ICD-10标准填写"],
    "确诊诊断": [],
    "鉴别诊断": []
  }
}
```

处理规则：
1. 当遇到"不详""未提供"等表述时留空
2. 过敏史需区分"无过敏史"和"未提及"
3. 手术史需包含手术名称和时间
4. 药品用法需包含"口服/外用/静脉滴注"等给药途径
5. 影像检查需注明"DR/CT/MRI"等检查类型

请处理以下门诊病历文本：
""" + record_text

        # 调用大模型获取结构化结果
        result = chat(prompt)
        #strip```json和```
        result = result.strip("```json\n")

        try:
            # 尝试解析返回的JSON
            return json.loads(result)
        except json.JSONDecodeError:
            # 如果解析失败，返回错误信息
            return {"error": "Failed to parse model response", "raw_response": result}


if __name__ == "__main__":
    # 测试用例
    test_record = """
    男性 汉族 
 78岁 农民 
 已婚 
 2022年01月03日11时59分 2022年01月03日15时48分 
 患者女儿 步行     主  诉：上腹部疼痛1天
    现病史：患者1天前无明显诱因出现上腹部疼痛，呈持续性疼痛，无呕吐，无畏寒、发热，无皮肤及巩膜黄染。患者为求诊治遂至门诊就诊，患者既往上腹部CT提示胆总管末端结石并胆系扩张。门诊遂拟以“胆总管结石”收入我院我科。
    患者自起病以来，精神较差，食欲不佳，睡眠一般，大小便正常，体重无明显改变。
    既往史：患者既往高血压病病史、冠状动脉心脏病病史。9月前外院行腹腔镜下胆囊切除术手术史。否认“糖尿病”等慢性疾病，否认“脑梗”等心脑血管疾病，否认“肝炎、结核、新冠”等传染性疾病，否认其他重大手术外伤输血史，否认食物药物过敏史，预防接种史不详。
    个人史：出生于原籍，无外地久居史。无血吸虫疫水接触史。否认吸烟史、否认饮酒史。无粉尘接触史。否认毒物或放射线接触史。无重大精神创伤史。否认冶游史
     婚育史：适龄结婚,育有子女，配偶及子女均体健。
    家族史：父母、兄弟、姐妹健康状况，无与患者类似疾病，无家族遗传倾向的疾病。
体 格 检 查
 T:36.8℃，P:92次/分，R:20次/分，BP:163/79mmHg
 发育正常，营养中等，急性面容，神志清楚，精神较差，自动体位，查体合作，对答切题。全身皮肤粘膜无黄染，未见皮疹及出血点，未见肝掌，未见蜘蛛痣。全身浅表淋巴结未触及肿大。头颅无畸形，五官端正；眼睑无水肿，双眼球运动自如，眼结膜无充血，双侧巩膜无黄染，双侧瞳孔等大等圆，对光反射灵敏；耳廓正常，无外耳道异常分泌物，耳突无压痛，听力无障碍，无鼻翼扇动，双鼻腔无异常分泌物，副鼻窦区无压痛。口角无歪斜，口唇无绀，口腔粘膜无破溃，口腔气味正常，牙齿正常，牙龈正常，伸舌居中，扁桃体双侧无肿大，咽无充血，声音正常。颈软，颈动脉搏动正常，颈动脉无杂音，颈静脉正常，肝颈静脉回流征阴性，气管居中，甲状腺无肿大，血管杂音无。胸廓对称，无胸骨叩痛，乳房正常。呼吸运动对称，肋间隙正常，呼吸动度一致，呼吸语颤对称，无胸膜摩擦音，双肺叩诊呈清音，肺下界位于锁骨中线第七肋间隙，肺下缘移动度16cm，双肺呼吸音清，未闻及干湿性啰音，无胸膜摩擦音，语音传导正常，心前区无隆起，心尖搏动正常，触诊无震颤，心脏相对浊音界正常，心率92次/分，律齐，各瓣膜听诊区未闻及病理性杂音，无心包摩擦音。腹平坦，双侧对称，呼吸运动正常，腹壁静脉无曲张，未见胃肠型、肠蠕动波，腹肌稍紧张，上腹部轻度压痛、无反跳痛，未触及肿物及包块，无液波震颤。肝于右锁骨中线肋缘下及剑突下未触及，脾于左肋缘下未触及，墨菲氏征阴性，肝上界于右锁骨中线上第五肋间叩出，肝浊音界正常，肝下界位于右肋缘下，肝脾肾区无叩痛，移动性浊音阴性；肠鸣音14次/分，无血管杂音，无振水音， 肛门及外生殖器未查。脊柱正常，无棘突压痛，活动度正常，四肢活动自如，关节无红肿，双下肢无水肿、无静脉曲张，肌力肌张力正常。生理反射存在,巴氏征阴性, 克氏征阴性,布氏征阴性。
    专科检查：腹平坦，双侧对称，呼吸运动正常，腹壁静脉无曲张，未见胃肠型、肠蠕动波，腹肌稍紧张，上腹部轻度压痛、无反跳痛，未触及肿物及包块，无液波震颤。肝于右锁骨中线肋缘下及剑突下未触及，脾于左肋缘下未触及，墨菲氏征阴性，肝上界于右锁骨中线上第五肋间叩出，肝浊音界正常，肝下界位于右肋缘下，肝脾肾区无叩痛，移动性浊音阴性；肠鸣音4次/分，无血管杂音，无振水音。
    辅助检查：上腹部CT平扫+增强（2021.10.05 我院）：1.胆总管末端结石并胆系扩张。2.胆囊切除术后。3.十二指肠降段憩室。
下腹部CT平扫+增强（2021.10.05 我院）：1.左输尿管膀胱壁内段结石，并左侧输尿管、左肾积水，并左肾周、输尿管周围少许渗出。2.右肾小结石。3.左肾囊肿可能。

  入院诊断：  1.急性胰腺炎
 2.胆总管结石伴胆管炎
 3.胆囊结石并胆囊炎
 4.冠状动脉性心脏病
 5.高血压病
 6.前列腺增生
 7.泌尿系结石
    """
    
    parser = MedicalRecordParser()
    result = parser.parse_medical_record(test_record)
    print(json.dumps(result, indent=2, ensure_ascii=False))